#include "viewer.h"

#include "trackball/glTrackball.h"
#include "scene.h"
#include "shadow.h"
#include "lightmanager.h"

CViewer* CViewer::m_pInstance = NULL;

CViewer::CViewer()
{
}

CViewer::~CViewer()
{
}

void CViewer::destroyViewer()
{
	if(m_pInstance)
		delete m_pInstance;
	m_pInstance = NULL;
}

CViewer* CViewer::getViewer()
{
	if(m_pInstance == NULL)
		m_pInstance = new CViewer();

	return m_pInstance;
}

void CViewer::draw()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
	glPushMatrix();
	GLTrackball::getTrackball()->setGLView();
	
	// get the model view transformation and compute its transposed inverse; then store it in texture matrix 6
	//TODO: see exercise sheet 4

	CLightManager::getLightManager()->positionSceneAlignedLights();
	CLightManager::getLightManager()->drawSceneAlignedLights();

	GLfloat color[] = {1.0, 1.0, 1.0, 1.0}; 
	GLfloat ambientcolor[] = {0.0, 0.0, 0.0, 1.0}; 
	glMaterialfv(GL_FRONT, GL_AMBIENT, ambientcolor);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
	glMaterialfv(GL_FRONT, GL_SPECULAR, color);
	glMaterialf(GL_FRONT, GL_SHININESS, 10);

	const std::list<CMesh*>& listMeshes = CScene::getScene()->getMeshes();
	std::list<CMesh*>::const_iterator meshIter;

	if(CScene::getScene()->getShadowsEnabled())
	{
		CLightManager::getLightManager()->drawSceneWithShadows();
	}
	else
	{
		CScene::getScene()->drawMeshes();
	}

	glPopMatrix();
	glutSwapBuffers();
}

void CViewer::initialize(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL | GLUT_RGB);
	glutInitWindowSize(800, 600);
	GLTrackball::getTrackball()->setWindowDimensions(800, 600);
	glutCreateWindow(argv[0]);

	gluLookAt(0, 0, -15, 0, 0, 0, 0, 1, 0);

  //read back the view transform to store it in the trackball
	GLTrackball::getTrackball()->setViewTransform();

	//glClearColor(1.0f, 1.0f, 1.0f, 1.0f);					// white background
	glClearColor(Globals::clearColor[0], Globals::clearColor[1], Globals::clearColor[2], Globals::clearColor[3]);					// white background
	glClearDepth(1000.0f);										// depth buffer setup
	glDepthFunc(GL_LEQUAL);										// the type of depth testing (less or equal)
	glEnable(GL_DEPTH_TEST);									// enable depth testing
	glShadeModel(GL_SMOOTH);									// employ gouraud shading	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// set perspective calculations to most accurate

	glEnable(GL_LIGHTING);										// enable lighting

	// enable backface-culling
	glEnable(GL_CULL_FACE);
	glFrontFace(GL_CCW);
	glCullFace(GL_BACK);

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);			// polygons should be filled

	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glPointSize(20);

	// --------------------------------------------------
	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		exit(-1);
	}
	// --------------------------------------------------

	CLightManager* lightManager = CLightManager::getLightManager();
	int l0 = lightManager->addPointLight(gmVector4(0.0, 5.0, 0.1, 1.0), gmVector3(0.0, 0.0, 0.0), gmVector3(0.0, 1.0, 0.0), gmVector3(1.0, 1.0, 1.0));
	lightManager->setHasShadow(l0, true);
}

void CViewer::loop()
{
	// specify that should be called in the glut main loop
	glutDisplayFunc(StaticViewer::display);
	glutReshapeFunc(StaticViewer::reshape);
	glutMouseFunc(StaticViewer::handleMouseClick);
	glutMotionFunc(StaticViewer::handleMouseMove);

	glutMainLoop();
}

namespace StaticViewer
{
	void display()
	{
		CViewer::getViewer()->draw();
	}

	void reshape(int w, int h)
	{
		// return, if width or hight of the window is zero
		if(w == 0 || h == 0) return;

		// adjust the size of the OpenGL viewport and frustum
		glViewport(0, 0, (GLsizei) w, (GLsizei) h);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(45, w / h, 1.0, 1001.0);		

		// switch back to modelview matrix stack
		glMatrixMode(GL_MODELVIEW);

		// notify the trackball about the window adjustment
		GLTrackball::getTrackball()->setWindowDimensions(w, h);
	}

	void handleMouseClick(int button, int state, int x, int y)
	{
		if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
		{
			GLTrackball::getTrackball()->mouseButton(0, 0, x, y);
		}
	}

	void handleMouseMove(int x, int y)
	{
		GLTrackball::getTrackball()->mouseMove(x, y);
		glutPostRedisplay();
	}
}
